package edu.sc.seis.TauP;

import javax.swing.*;
import java.util.Vector;
import java.awt.*;

/*
  The TauP Toolkit: Flexible Seismic Travel-Time and Raypath Utilities.
  Copyright (C) 1998-2000 University of South Carolina

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  The current version can be found at 
  <A HREF="www.seis.sc.edu">http://www.seis.sc.edu</A>

  Bug reports and comments should be directed to 
  H. Philip Crotwell, crotwell@seis.sc.edu or
  Tom Owens, owens@seis.sc.edu

*/

/**
 * PathPlot.java
 *
 *
 * Created: Fri May  7 15:45:43 1999
 *
 * @author Philip Crotwell
 * @version 1.1 Wed Feb  2 20:40:49 GMT 2000
 */

public class PathPlot extends JPanel {
    
    public PathPlot() {
	//	setDebugGraphicsOptions(DebugGraphics.LOG_OPTION);
	setOpaque(true);
	//	setBackground(java.awt.Color.white);
    }
    
    public void setSelectedIndex(int index) {
	this.selectedIndex = index;
	repaint();
    }

    public void addElement(Arrival a) {
	arrivals.addElement(a);
    }

    public void removeAllElements() {
	arrivals.removeAllElements();
	repaint();
    }

    public void setTauModel(TauModel tMod) {
	this.tMod = tMod;
    }

    public void paintComponent(Graphics g) {
	super.paintComponent(g);

	Arrival a;
	int[] x, y;
	int xOffset = getSize().width/2;
	int yOffset = getSize().height/2;
	int pixelRad = Math.min(xOffset,
				yOffset);
	double roe = 6371;
	Color aColor, bColor, fillColor;
	aColor = new Color(220, 220, 220);
	bColor = Color.white;
	fillColor = aColor;
	boolean whichColor = true;

	int disconRad;
	if (tMod != null) {
	    roe = tMod.getRadiusOfEarth();
	    disconRad =  pixelRad;
	    fillColor = whichColor ? aColor : bColor;
	    g.setColor(fillColor);
	    whichColor = ! whichColor;
	    g.fillOval(xOffset-disconRad, yOffset-disconRad,
		       2*disconRad,
		       2*disconRad);
	    for (int i=0;i<tMod.tauBranches[0].length-1;i++) {
		if (tMod.tauBranches[0][i].getBotDepth() !=
		    tMod.tauBranches[1][i].getBotDepth()) {
		    disconRad = (int)Math.round((tMod.getRadiusOfEarth()-
						 tMod.tauBranches[0][i].getBotDepth()) / 
						roe * pixelRad);
		    fillColor = whichColor ? aColor : bColor;
		    g.setColor(fillColor);
		    whichColor = ! whichColor;
		    g.fillOval(xOffset-disconRad, yOffset-disconRad,
			       2*disconRad,
			       2*disconRad);
		} else {
		    disconRad = (int)Math.round((tMod.getRadiusOfEarth()-
						 tMod.tauBranches[1][i].getBotDepth()) / 
						roe * pixelRad);
		    fillColor = whichColor ? aColor : bColor;
		    g.setColor(fillColor);
		    whichColor = ! whichColor;
		    g.fillOval(xOffset-disconRad, yOffset-disconRad,
			       2*disconRad,
			       2*disconRad);
		}
	    }
	}

	g.setColor(Color.black);

	for (int i=0;i<arrivals.size(); i++) {
	    a = (Arrival)arrivals.elementAt(i);
	    x = new int[a.getNumPathPoints()];
	    y = new int[a.getNumPathPoints()];
	    for (int j = 0; j < x.length; j++) {
		x[j] = xOffset +(int)Math.rint(Math.sin(a.getPathPoint(j).dist)
                                               * ( roe-a.getPathPoint(j).depth) / 
					       roe * pixelRad);
		y[j] = yOffset -(int)Math.rint(Math.cos(a.getPathPoint(j).dist)
                                               * ( roe-a.getPathPoint(j).depth) / 
					       roe * pixelRad);
	    }
	    if (i==selectedIndex) {
		g.setColor(Color.red);
	    }		
	    g.drawPolyline(x, y, x.length);
	    if (i==selectedIndex) {
		g.setColor(Color.black);
	    }
	}
    }

    protected Vector arrivals = new Vector();
    protected TauModel tMod;
    private int selectedIndex = 0;

} // PathPlot
